;; Generated package description from ivy-hydra.el  -*- mode: lisp-data; no-byte-compile: t -*-
(define-package "ivy-hydra" "0.15.0" "Additional key bindings for Ivy" '((emacs "24.5") (ivy "0.15.0") (hydra "0.14.0")) :commit "ffde82793eb2bfeb552154ed5f6e584ed9f064cd" :authors '(("Oleh Krehel" . "ohwoeowho@gmail.com")) :maintainer '("Basil L. Contovounesios" . "basil@contovou.net") :keywords '("convenience") :url "https://github.com/abo-abo/swiper")
